import express from 'express'
import { createSchema, editSchema } from '../schema/room'
import Room, { IRoom } from '../models/room'

const router = express.Router()

/**
 * @swagger
 * /room/{id}:
 *   get:
 *     tags:
 *       - Room
 *     room: Fetch data by id
 *     parameters:
 *       - name: id
 *         in: path
 *         required: true
 *         description: ID of the room
 *         schema:
 *           type: string
 *     responses:
 *       200:
 *         description: Data
 *       400:
 *         description: Bad request
 *       404:
 *         description: Not found
 *       500:
 *         description: Internal server error
 */
router.get("/emergency", async (req, res) => {
  try {
    // console.log("test")
    const room = await Room.find({
      "service.gasDetector": true
    })

    if (room.length > 0) {
      return res.status(200).send(true)
    }
    return res.status(200).send(false)
  } catch (error) {
    return res.status(500).send(error)
  }
}
)
router.get('/:name', async (req, res) => {
  try {
    // console.log(req.params?.name)
    const name = req.params?.name
    if (!name) {
      return res.status(400).send('Bad request')
    }
    const allTask = []
    allTask.push(Room.findOne({ name }))
    allTask.push(Room.findOne({"service.gasDetector": true}))
    const [room, emergency] = await Promise.all(allTask)
   
    // let emergency = Room.findOne({
    //   "service.gasDetector": true
    // })

    if (room) {
      room.service.gasDetector = emergency ? true : false
      return res.status(200).send(room)
    }

    return res.status(404).send('Not found')
  } catch (error) {
    return res.status(500).send(error)
  }
})

/**
 * @swagger
 * /room/create:
 *   post:
 *     tags:
 *       - Room
 *     room: Create a new room
 *     requestBody:
 *       required: true
 *       content:
 *         application/json:
 *           schema:
 *             type: object
 *             properties:
 *               name:
 *                 type: string
 *                 description: Name of the room
 *             required: [name]
 *     responses:
 *       200:
 *         description: Created
 *       400:
 *         description: Bad request
 *       404:
 *         description: Not found
 *       500:
 *         description: Internal server error
 */
router.post('/create', async (req, res) => {
  try {
    const createData = <IRoom>req.body
   
    const result = await Room.create({
      ...createData,
    })

    if (result) {
      return res.status(200).send(result)
    }

    return res.status(500).send('Failed to create room')
  } catch (error) {
    return res.status(500).send(error)
  }
})

/**
 * @swagger
 * /room/edit:
 *   put:
 *     tags:
 *       - Room
 *     room: Edit room data
 *     requestBody:
 *       required: true
 *       content:
 *        application/json:
 *           schema:
 *             type: object
 *             properties:
 *               name:
 *                 type: string
 *                 description: Name of the room
 *               light:
 *                 type: boolean
 *                 description: Light status of the room
 *             required: [name, light]
 *     responses:
 *       200:
 *         description: Edited
 *       400:
 *         description: Bad request
 *       404:
 *         description: Not found
 *       500:
 *         description: Internal server error
 */
interface EditService {
  door?: boolean
  light?: boolean
  airConditioner?: boolean
  temperature?: number
  gasDetector?: boolean
  [key: string]: any
}
interface EditIRoom {
  name: string
  service: EditService
}
router.put('/edit', async (req, res) => {
  try {
    // console.log(req.body)
    const editData = <EditIRoom>req.body
    
    const result  = await Room.findOne({ name: editData.name })

    if (result) {
      result.service = {
        ...result.service._doc,...editData.service
      }
      await result.save()
      return res.status(200).send(result)
    }

    return res.status(404).send('Not found')
  } catch (error) {
    console.log(error)
    return res.status(500).send(error)
  }
})

export default router
